#ifndef GETPIXEL_C
#define GETPIXEL_C

#include <stdlib.h>

#ifndef NAN
#define NAN (__builtin_nanf(""))
#endif

typedef float (*getsample_operator)(float*,int,int,int,int,int,int);
//typedef void (*setsample_operator)(float*,int,int,int,int,int,int,float);

// extrapolate by 0

float getsample_0(float *x, int w, int h, int pd, int i, int j, int l);

// extrapolate by nearest value
float getsample_1(float *x, int w, int h, int pd, int i, int j, int l);

// extrapolate by nan
float getsample_nan(float *x, int w, int h, int pd, int i, int j, int l);

// force a segfault if extrapolation is required
float getsample_error(float *x, int w, int h, int pd, int i, int j, int l);

// abort the program extrapolation is required
float getsample_abort(float *x, int w, int h, int pd, int i, int j, int l);

// exit the program extrapolation is required
float getsample_exit(float *x, int w, int h, int pd, int i, int j, int l);

// like n%p, but works for all numbers
int gmod(int x, int m);

int positive_reflex(int n, int p);

// extrapolate by reflection
float getsample_2(float *x, int w, int h, int pd, int i, int j, int l);

// extrapolate by periodicity
float getsample_per(float *x, int w, int h, int pd, int i, int j, int l);

// extrapolate by constant (set by calling it with zero sizes)
float getsample_constant(float *x, int w, int h, int pd, int i, int j, int l);


// test for inclusion of stdlib.h and string.h
#if defined(EXIT_SUCCESS)
#if defined(_STRING_H) || defined(_STRING_H_)
getsample_operator get_sample_operator(getsample_operator o)
{
	static char *option = NULL;
	static int env_getpixel_checked = 0;
	if (!env_getpixel_checked)
	{
		env_getpixel_checked = 1;
		option = getenv("GETPIXEL");
	}
	if (!option) return o;
	if (0 == strcmp(option, "nan"      ))  return getsample_nan;
	if (0 == strcmp(option, "segfault" ))  return getsample_error;
	if (0 == strcmp(option, "error"    ))  return getsample_error;
	if (0 == strcmp(option, "abort"    ))  return getsample_abort;
	if (0 == strcmp(option, "exit"    ))   return getsample_exit;
	if (0 == strcmp(option, "periodic" ))  return getsample_per;
	if (0 == strcmp(option, "constant" ))  return getsample_0;
	if (0 == strcmp(option, "nearest"   )) return getsample_1;
	if (0 == strcmp(option, "reflex"   ))  return getsample_2;
	if (0 == strcmp(option, "symmetric"))  return getsample_2;
	char *endptr;
	float value = strtof(option, &endptr);
	if (endptr != option) {
		getsample_constant(&value, 0, 0, 0, 0, 0, 0);
		return getsample_constant;
	}
	return getsample_0;
}
#endif//_STRING_H
#endif//EXIT_SUCCESS


void setsample_0(float *x, int w, int h, int pd, int i, int j, int l,
		float v);

void setsample_segf(float *x, int w, int h, int pd, int i, int j, int l,
		float v);

typedef float (*getpixel_operator)(float*,int,int,int,int);

float getpixel_0(float *x, int w, int h, int i, int j);

float getpixel_1(float *x, int w, int h, int i, int j);

//
//static void setpixel(float *x, int w, int h, int i, int j, float v)
//{
//	if (i < 0 || i >= w || j < 0 || j >= h)
//		return;
//	x[i + j*w] = v;
//}

#endif//GETPIXEL_C
