#ifndef _RANDOM_C
#define _RANDOM_C

#include <math.h>
#include <stdlib.h>
#include <limits.h>
#include <stdint.h>


// Why write a simple random number generator here?
// It is the easiest way to fullfill the following requirements
// 	1. very fast, negligible memory usage
// 	2. generates the same sequence on any architecture
// 	3. do not have annoying warnings (e.g. unsafe rand() in openbsd).
// Strong non-requirements:
// 	1. cryptographicaly secure
// 	2. pass fancy statistical tests


void lcg_knuth_srand(uint32_t x);

uint32_t lcg_knuth_rand(void);


void xsrand(unsigned int seed);

int xrand(void);

// warning: the low bits will be set to zero (!) when converting to float
double random_raw(void);

double random_uniform(void);

double random_ramp(void);

#ifndef M_PI
#define M_PI 3.14159265358979323846264338328
#endif

double random_normal(void);

int randombounds(int a, int b);

double random_laplace(void);

double random_cauchy(void);

double random_exponential(void);

double random_pareto(void);

// This function samples a stable variable of parameters (alpha,beta)
//
// The algorithm is copied verbatim from formulas (2.3) and (2.4) in
//     Chambers, J. M.; Mallows, C. L.; Stuck, B. W. (1976).
//     "A Method for Simulating Stable Random Variables".
//     Journal of the American Statistical Association 71 (354): 340–344.
//     doi:10.1080/01621459.1976.10480344.
//
// Observation: the algorithm is numerically imprecise when alpha approaches 1.
// TODO: implement appropriate rearrangements as suggested in the article.
double random_stable(double alpha, double beta);

#endif//_RANDOM_C
