#!/usr/bin/env octave-cli

% parse command line
args = argv();
if nargin ~= 6 && nargin ~= 7,
    error(sprintf(['Incorrect number of arguments.\n' ...
        'Usage: mlri-demosaic.m input output wb-red wb-green wb-blue gamma [black-level]\n']));
end
input  = args{1};
output = args{2};
fr = str2double(args{3});
fg = str2double(args{4});
fb = str2double(args{5});
gamma = str2double(args{6});
if nargin == 7, black = str2double(args{7}); else black = 1600; end;

pkg load image % required Octave packages
addpath(fileparts(mfilename('fullpath')));

pattern = 'rggb';

% black level
raw = double(iio_read(input)) - black;

% white balance
if size(raw,3) == 1,
	raw(1:2:end,1:2:end) = fr*raw(1:2:end,1:2:end);
	raw(2:2:end,1:2:end) = fg*raw(2:2:end,1:2:end);
	raw(1:2:end,2:2:end) = fg*raw(1:2:end,2:2:end);
	raw(2:2:end,2:2:end) = fb*raw(2:2:end,2:2:end);
else % bayer pattern arranged as 4-channels
	raw(:,:,1) = fg*raw(:,:,1);
	raw(:,:,2) = fr*raw(:,:,2);
	raw(:,:,3) = fb*raw(:,:,3);
	raw(:,:,4) = fg*raw(:,:,4);
end

% saturate
raw = raw/prctile(raw(:),99.5);

% demosaick
%rgb = demosaick(raw, pattern, 1.4);
rgb = demosaick(255*min(1,raw), pattern, 1.4);
%iio_write(output, rgb);

% apply gamma
iio_write(output, 255*(rgb/255).^(1/gamma));
