from .base_options import BaseOptions


class TrainOptions(BaseOptions):
    """This class includes training options.

    It also includes shared options defined in BaseOptions.
    """

    def initialize(self, parser):
        parser = BaseOptions.initialize(self, parser)
        # visdom and HTML visualization parameters        
        parser.add_argument('--display_freq', type=int, default=400, help='frequency of showing training results on screen')
        
        parser.add_argument('--update_freq', type=int, default=1000, help='frequency of saving training results to html')
        parser.add_argument('--print_freq', type=int, default=100, help='frequency of showing training results on console')
    
        # network saving and loading parameters
        parser.add_argument('--save_latest_freq', type=int, default=5000, help='frequency of saving the latest results')
        parser.add_argument('--save_epoch_freq', type=int, default=1, help='frequency of saving checkpoints at the end of epochs')
        parser.add_argument('--save_by_iter', action='store_true', help='whether saves model by iteration')
        parser.add_argument('--continue_train', default=False, action='store_true', help='continue training: load the latest model')
        parser.add_argument('--autoresume', default=False, action='store_true', help='continue training: load and keep track of epochs')
        parser.add_argument('--epoch_count', type=int, default=1, help='the starting epoch count, we save the model by <epoch_count>, <epoch_count>+<save_latest_freq>, ...')
        parser.add_argument('--phase', type=str, default='train', help='train, val, test, etc')
        parser.add_argument('--path2epoch', type=str, default='', help='loads in preference networks weights stored at: /path/to/folder/epoch. It loads at the end /path/to/folder/epoch_net_$(name).pth.')

        # training parameters
        parser.add_argument('--niter', type=int, default=5, help='# of iter at starting learning rate')
        parser.add_argument('--niter_decay', type=int, default=5, help='# of iter to linearly decay learning rate to zero')
        parser.add_argument('--beta1', type=float, default=0.5, help='momentum term of adam')
        parser.add_argument('--lr', type=float, default=0.0002, help='initial learning rate for adam')
        parser.add_argument('--pool_size', type=int, default=50, help='the size of image buffer that stores previously generated images')
        parser.add_argument('--lr_policy', type=str, default='linear', help='learning rate policy. [linear | step | plateau | cosine]')
        parser.add_argument('--lr_decay_iters', type=int, default=50, help='multiply by a gamma every lr_decay_iters iterations')
        # validation datasets
        parser.add_argument('--val_dataroot', type=str, default='./datasets/validation_dataset', required=False, help='path to images (should have subfolders trainA, trainB, valA, valB, etc)')
        parser.add_argument('--val_dataset_mode', type=str, default='inferaxel', help='chooses how datasets are loaded. [ axel | inferaxel | dummy ]')
        parser.add_argument('--val_max_dataset_size', type=int, default=float("inf"), help='Maximum number of samples allowed per dataset. If the dataset directory contains more than max_dataset_size, only a subset is loaded.')
        parser.add_argument('--val_videos',  type=str, default=None, help='which videos to use for testing.') #Only used in inferaxel.
        parser.add_argument('--no_val', action='store_true', default=False, help='Dont test on val data while training.')
        parser.add_argument('--val_visuals', type=str, default='None', help='[none, fusion].') 
        parser.add_argument('--val_epoch_freq', type=int, default=1, help='frequency (in epochs) for computing the validation')

        self.isTrain = True
        return parser
